@@ -54,23 +54,13 @@ static void adapt_store_registers ();
 static void adapt_close ();
 static int  adapt_clear_breakpoints();
 
-/* 
- * Processor types. It is assumed that the adapt has the correct 
- * ROM for the given processor. 
- */
-#define TYPE_UNKNOWN	0
-#define TYPE_A29000	1
-#define TYPE_A29030	2
-#define TYPE_A29050	3
-static char *processor_name[] = { "Unknown", "A29000", "A29030", "A29050" };
-static int processor_type=TYPE_UNKNOWN;
-
 #define FREEZE_MODE 	(read_register(CPS_REGNUM) && 0x400) 
-#define USE_SHADOW_PC   ((processor_type == TYPE_A29050) && FREEZE_MODE)
+#define USE_SHADOW_PC	((processor_type == a29k_freeze_mode) && FREEZE_MODE)
 
 /* Can't seem to get binary coff working */
 #define ASCII_COFF		/* Adapt will be downloaded with ascii coff */
 
+/* FIXME: Replace with `set remotedebug'.  */
 #define LOG_FILE "adapt.log"
 #if defined (LOG_FILE)
 FILE *log_file=NULL;
@@ -417,7 +407,7 @@ adapt_create_inferior (execfile, args, env)
   	  expect_prompt ();
 #else
   	insert_breakpoints ();  /* Needed to get correct instruction in cache */
-  	proceed(entry_pt, -1, 0);
+  	proceed(entry_pt, TARGET_SIGNAL_DEFAULT, 0);
 #endif
 
   } else {
@@ -570,28 +560,12 @@ the baud rate, and the name of the program to run on the remote system.");
   /* Clear any break points */
   adapt_clear_breakpoints();
 
-  /* Determine the processor revision level */
-  prl = (unsigned int)read_register(CFG_REGNUM) >> 24;
-  if (prl == 0x03) { 
-	processor_type = TYPE_A29000;  
-  } else if ((prl&0xf0) == 0x40) { 	/* 29030 = 0x4* */
-	processor_type = TYPE_A29030;  
-	fprintf_filtered(stderr,"WARNING: debugging of A29030 not tested.\n");
-  } else if ((prl&0xf0) == 0x20) { 	/* 29050 = 0x2* */
-	processor_type = TYPE_A29050;  
-	fprintf_filtered(stderr,"WARNING: debugging of A29050 not tested.\n");
-  } else {
-	processor_type = TYPE_UNKNOWN;  
-	fprintf_filtered(stderr,"WARNING: processor type unknown.\n");
-  }
-
   /* Print out some stuff, letting the user now what's going on */
-  printf_filtered("Remote debugging on an %s connect to an Adapt via %s.\n",
-		processor_name[processor_type],dev_name);
+  printf_filtered("Connected to an Adapt via %s.\n", dev_name);
     /* FIXME: can this restriction be removed? */
   printf_filtered("Remote debugging using virtual addresses works only\n");
   printf_filtered("\twhen virtual addresses map 1:1 to physical addresses.\n"); 
-  if (processor_type != TYPE_A29050) {
+  if (processor_type != a29k_freeze_mode) {
 	fprintf_filtered(stderr,
 	"Freeze-mode debugging not available, and can only be done on an A29050.\n");
   }
@@ -676,7 +650,8 @@ adapt_detach (args,from_tty)
 
 void
 adapt_resume (pid, step, sig)
-     int pid, step, sig;
+     int pid, step;
+     enum target_signal sig;
 {
   if (step)	
     {
@@ -706,7 +681,7 @@ adapt_resume (pid, step, sig)
 
 int
 adapt_wait (status)
-     WAITTYPE *status;
+     struct target_waitstatus *status;
 {
   /* Strings to look for.  '?' means match any single character.  
      Note that with the algorithm we use, the initial character
@@ -731,11 +706,13 @@ adapt_wait (status)
   int old_timeout = timeout;
   int old_immediate_quit = immediate_quit;
 
-  WSETEXIT ((*status), 0);
+  status->kind = TARGET_WAITKIND_EXITED;
+  status->value.integer = 0;
 
   if (need_artificial_trap != 0)
     {
-      WSETSTOP ((*status), SIGTRAP);
+      status->kind = TARGET_WAITKIND_STOPPED;
+      status->value.sig = TARGET_SIGNAL_TRAP;
       need_artificial_trap--;
       return 0;
     }
@@ -776,9 +753,15 @@ adapt_wait (status)
   }
   expect_prompt ();
   if (*bp== '\0')
-    WSETSTOP ((*status), SIGTRAP);
+    {
+      status->kind = TARGET_WAITKIND_STOPPED;
+      status->value.sig = TARGET_SIGNAL_TRAP;
+    }
   else
-    WSETEXIT ((*status), 0);
+    {
+      status->kind = TARGET_WAITKIND_EXITED;
+      status->value.integer = 0;
+    }
   timeout = old_timeout;
   immediate_quit = old_immediate_quit;
   return 0;
@@ -1331,7 +1314,7 @@ adapt_com (args, fromtty)
 		break;
 	/* Begin commands that take input in the form 'c x,y[,z...]' */
 	case 'S':	/* Set memory or register */
-		if (index(args,',')) {	/* Assume it is properly formatted */
+		if (strchr(args,',')) {	/* Assume it is properly formatted */
 			write(adapt_desc,args,strlen(args));
 			write(adapt_desc,"\r",1);
 			expect_prompt();
