@@ -1,5 +1,5 @@
 /* Symbol table definitions for GDB.
-   Copyright (C) 1986, 1989, 1991, 1992 Free Software Foundation, Inc.
+   Copyright 1986, 1989, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GDB.
 
@@ -26,10 +26,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define obstack_chunk_alloc xmalloc
 #define obstack_chunk_free free
 
+/* GNU C supports enums that are bitfields.  Some old compilers don't. */
+#if defined(__GNUC__) && !defined(BYTE_BITFIELD)
+#define	BYTE_BITFIELD	:8;
+#else
+#define	BYTE_BITFIELD	/*nothing*/
+#endif
+
 /* Define a structure for the information that is common to all symbol types,
    including minimal symbols, partial symbols, and full symbols.  In a
    multilanguage environment, some language specific information may need to
-   be recorded along with each symbol. */
+   be recorded along with each symbol.
+
+   These fields are ordered to encourage good packing, since we frequently
+   have tens or hundreds of thousands of these.  */
 
 struct general_symbol_info
 {
@@ -61,12 +71,6 @@ struct general_symbol_info
     }
   value;
 
-  /* Record the source code language that applies to this symbol.
-     This is used to select one of the fields from the language specific
-     union below. */
-
-  enum language language;
-
   /* Since one and only one language can apply, wrap the language specific
      information inside a union. */
 
@@ -82,13 +86,20 @@ struct general_symbol_info
 	} chill_specific;
     } language_specific;
 
+  /* Record the source code language that applies to this symbol.
+     This is used to select one of the fields from the language specific
+     union above. */
+
+  enum language language BYTE_BITFIELD;
+
   /* Which section is this symbol in?  This is an index into
      section_offsets for this objfile.  Negative means that the symbol
      does not get relocated relative to a section.
-     Disclaimer: currently this is just used for xcoff, so don't expect
-     all symbol-reading code to set it correctly.  */
+     Disclaimer: currently this is just used for xcoff, so don't
+     expect all symbol-reading code to set it correctly (the ELF code
+     also tries to set it correctly).  */
 
-  int section;
+  short section;
 };
 
 #define SYMBOL_NAME(symbol)		(symbol)->ginfo.name
@@ -293,7 +304,7 @@ struct minimal_symbol
       mst_file_text,		/* Static version of mst_text */
       mst_file_data,		/* Static version of mst_data */
       mst_file_bss		/* Static version of mst_bss */
-    } type;
+    } type BYTE_BITFIELD;
 
 };
 
@@ -515,6 +526,25 @@ enum address_class
 
   LOC_LOCAL_ARG,
 
+  /* Value is at SYMBOL_VALUE offset from the current value of
+     register number SYMBOL_BASEREG.  This exists mainly for the same
+     things that LOC_LOCAL and LOC_ARG do; but we need to do this
+     instead because on 88k DWARF gives us the offset from the
+     frame/stack pointer, rather than the offset from the "canonical
+     frame address" used by COFF, stabs, etc., and we don't know how
+     to convert between these until we start examining prologues.
+
+     Note that LOC_BASEREG is much less general than a DWARF expression.
+     We don't need the generality (at least not yet), and storing a general
+     DWARF expression would presumably take up more space than the existing
+     scheme.  */
+
+  LOC_BASEREG,
+
+  /* Same as LOC_BASEREG but it is an argument.  */
+
+  LOC_BASEREG_ARG,
+
   /* The variable does not actually exist in the program.
      The value is ignored.  */
 
@@ -528,18 +558,18 @@ struct symbol
 
   struct general_symbol_info ginfo;
 
-  /* Name space code.  */
-
-  enum namespace namespace;
-
-  /* Address class */
-
-  enum address_class class;
-
   /* Data type of value */
 
   struct type *type;
 
+  /* Name space code.  */
+
+  enum namespace namespace BYTE_BITFIELD;
+
+  /* Address class */
+
+  enum address_class class BYTE_BITFIELD;
+
   /* Line number of definition.  FIXME:  Should we really make the assumption
      that nobody will try to debug files longer than 64K lines?  What about
      machine generated programs? */
@@ -551,34 +581,17 @@ struct symbol
 
   union
     {
-      /* for OP_BASEREG in DWARF location specs */
-      struct
-	{
-	  short regno_valid;	/* 0 == regno invalid; !0 == regno valid */
-	  short regno;		/* base register number {0, 1, 2, ...} */
-	} basereg;
+      /* Used by LOC_BASEREG and LOC_BASEREG_ARG.  */
+      short basereg;
     }
   aux_value;
-
 };
 
 #define SYMBOL_NAMESPACE(symbol)	(symbol)->namespace
 #define SYMBOL_CLASS(symbol)		(symbol)->class
 #define SYMBOL_TYPE(symbol)		(symbol)->type
 #define SYMBOL_LINE(symbol)		(symbol)->line
-#define SYMBOL_BASEREG(symbol)		(symbol)->aux_value.basereg.regno
-
-/* This currently fails because some symbols are not being initialized
-   to zero on allocation, and no code is currently setting this value.
-   Basereg handling will probably change significantly in the next release.
-   FIXME -fnf */
-
-#if 0
-#define SYMBOL_BASEREG_VALID(symbol) (symbol)->aux_value.basereg.regno_valid
-#else
-#define SYMBOL_BASEREG_VALID(symbol) 0
-#endif
-
+#define SYMBOL_BASEREG(symbol)		(symbol)->aux_value.basereg
 
 /* A partial_symbol records the name, namespace, and address class of
    symbols whose types we have not parsed yet.  For functions, it also
@@ -596,11 +609,11 @@ struct partial_symbol
 
   /* Name space code.  */
 
-  enum namespace namespace;
+  enum namespace namespace BYTE_BITFIELD;
 
   /* Address class (for info_symbols) */
 
-  enum address_class class;
+  enum address_class class BYTE_BITFIELD;
 
 };
 
@@ -628,26 +641,27 @@ struct linetable_entry
   CORE_ADDR pc;
 };
 
-/* The order of entries in the linetable is significant.
+/* The order of entries in the linetable is significant.  They should
+   be sorted by increasing values of the pc field.  If there is more than
+   one entry for a given pc, then I'm not sure what should happen (and
+   I not sure whether we currently handle it the best way).
 
-   It should generally be in ascending line number order.  Line table
-   entries for a function at lines 10-40 should come before entries
-   for a function at lines 50-70.
-
-   A for statement looks like this
+   Example: a C for statement generally looks like this
 
    	10	0x100	- for the init/test part of a for stmt.
    	20	0x200
    	30	0x300
    	10	0x400	- for the increment part of a for stmt.
 
-   FIXME: this description is incomplete.  coffread.c is said to get
-   the linetable order wrong (would arrange_linenos from xcoffread.c
-   work for normal COFF too?).  */
+   */
 
 struct linetable
 {
   int nitems;
+
+  /* Actually NITEMS elements.  If you don't like this use of the
+     `struct hack', you can shove it up your ANSI (seriously, if the
+     committee tells us how to do it, we can probably go along).  */
   struct linetable_entry item[1];
 };
 
@@ -676,7 +690,7 @@ struct section_offsets
 
 #define	ANOFFSET(secoff, whichone)	(secoff->offsets[whichone])
 
-/* Each source file is represented by a struct symtab. 
+/* Each source file or header is represented by a struct symtab. 
    These objects are chained through the `next' field.  */
 
 struct symtab
@@ -686,12 +700,14 @@ struct symtab
 
     struct symtab *next;
 
-    /* List of all symbol scope blocks for this symtab.  */
+    /* List of all symbol scope blocks for this symtab.  May be shared
+       between different symtabs (and normally is for all the symtabs
+       in a given compilation unit).  */
 
     struct blockvector *blockvector;
 
     /* Table mapping core addresses to line numbers for this file.
-       Can be NULL if none.  */
+       Can be NULL if none.  Never shared between different symtabs.  */
 
     struct linetable *linetable;
 
@@ -718,7 +734,8 @@ struct symtab
        free_contents => do a tree walk and free each object.
        free_nothing => do nothing; some other symtab will free
          the data this one uses.
-      free_linetable => free just the linetable.  */
+      free_linetable => free just the linetable.  FIXME: Is this redundant
+      with the primary field?  */
 
     enum free_code
       {
@@ -970,17 +987,28 @@ contained_in PARAMS ((struct block *, struct block *));
 extern void
 reread_symbols PARAMS ((void));
 
+/* Macro for name of symbol to indicate a file compiled with gcc. */
+#ifndef GCC_COMPILED_FLAG_SYMBOL
+#define GCC_COMPILED_FLAG_SYMBOL "gcc_compiled."
+#endif
+
+/* Macro for name of symbol to indicate a file compiled with gcc2. */
+#ifndef GCC2_COMPILED_FLAG_SYMBOL
+#define GCC2_COMPILED_FLAG_SYMBOL "gcc2_compiled."
+#endif
+
 /* Functions for dealing with the minimal symbol table, really a misc
    address<->symbol mapping for things we don't have debug symbols for.  */
 
-extern void
-prim_record_minimal_symbol PARAMS ((const char *, CORE_ADDR,
-				    enum minimal_symbol_type));
+extern void prim_record_minimal_symbol PARAMS ((const char *, CORE_ADDR,
+						enum minimal_symbol_type,
+						struct objfile *));
 
-extern void
-prim_record_minimal_symbol_and_info PARAMS ((const char *, CORE_ADDR,
-					     enum minimal_symbol_type,
-					     char *info, int section));
+extern void prim_record_minimal_symbol_and_info
+  PARAMS ((const char *, CORE_ADDR,
+	   enum minimal_symbol_type,
+	   char *info, int section,
+	   struct objfile *));
 
 extern struct minimal_symbol *
 lookup_minimal_symbol PARAMS ((const char *, struct objfile *));
@@ -1022,13 +1050,21 @@ struct symtabs_and_lines
 extern struct symtab_and_line
 find_pc_line PARAMS ((CORE_ADDR, int));
 
+/* Given an address, return the nearest symbol at or below it in memory.
+   Optionally return the symtab it's from through 2nd arg, and the
+   address in inferior memory of the symbol through 3rd arg.  */
+
+extern struct symbol *
+find_addr_symbol PARAMS ((CORE_ADDR, struct symtab **, CORE_ADDR *));
+
 /* Given a symtab and line number, return the pc there.  */
 
 extern CORE_ADDR
 find_line_pc PARAMS ((struct symtab *, int));
 
 extern int 
-find_line_pc_range PARAMS ((struct symtab *, int, CORE_ADDR *, CORE_ADDR *));
+find_line_pc_range PARAMS ((struct symtab_and_line,
+			    CORE_ADDR *, CORE_ADDR *));
 
 extern void
 resolve_sal_pc PARAMS ((struct symtab_and_line *));
