@@ -29,7 +29,8 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include "signals.h"
 #include "gdbcmd.h"
-#include "terminal.h"
+#include "serial.h"
+#include "terminal.h" /* For job_control */
 #include "bfd.h"
 #include "target.h"
 #include "demangle.h"
@@ -70,8 +71,16 @@ static struct cleanup *cleanup_chain;
 
 int quit_flag;
 
-/* Nonzero means quit immediately if Control-C is typed now,
-   rather than waiting until QUIT is executed.  */
+/* Nonzero means quit immediately if Control-C is typed now, rather
+   than waiting until QUIT is executed.  Be careful in setting this;
+   code which executes with immediate_quit set has to be very careful
+   about being able to deal with being interrupted at any time.  It is
+   almost always better to use QUIT; the only exception I can think of
+   is being able to quit out of a system call (using EINTR loses if
+   the SIGINT happens between the previous QUIT and the system call).
+   To immediately quit in the case in which a SIGINT happens between
+   the previous QUIT and setting immediate_quit (desirable anytime we
+   expect to block), call QUIT after setting immediate_quit.  */
 
 int immediate_quit;
 
@@ -207,7 +216,7 @@ warning_setup ()
 {
   target_terminal_ours ();
   wrap_here("");			/* Force out any buffered output */
-  fflush (stdout);
+  gdb_flush (gdb_stdout);
 }
 
 /* Print a warning message.
@@ -227,12 +236,12 @@ warning (va_alist)
   va_start (args);
   target_terminal_ours ();
   wrap_here("");			/* Force out any buffered output */
-  fflush (stdout);
+  gdb_flush (gdb_stdout);
   if (warning_pre_print)
-    fprintf (stderr, warning_pre_print);
+    fprintf_unfiltered (gdb_stderr, warning_pre_print);
   string = va_arg (args, char *);
-  vfprintf (stderr, string, args);
-  fprintf (stderr, "\n");
+  vfprintf_unfiltered (gdb_stderr, string, args);
+  fprintf_unfiltered (gdb_stderr, "\n");
   va_end (args);
 }
 
@@ -251,14 +260,14 @@ error (va_alist)
   va_start (args);
   target_terminal_ours ();
   wrap_here("");			/* Force out any buffered output */
-  fflush (stdout);
+  gdb_flush (gdb_stdout);
   if (error_pre_print)
-    fprintf_filtered (stderr, error_pre_print);
+    fprintf_filtered (gdb_stderr, error_pre_print);
   string = va_arg (args, char *);
-  vfprintf_filtered (stderr, string, args);
-  fprintf_filtered (stderr, "\n");
+  vfprintf_filtered (gdb_stderr, string, args);
+  fprintf_filtered (gdb_stderr, "\n");
   va_end (args);
-  return_to_top_level ();
+  return_to_top_level (RETURN_ERROR);
 }
 
 /* Print an error message and exit reporting failure.
@@ -278,9 +287,9 @@ fatal (va_alist)
 
   va_start (args);
   string = va_arg (args, char *);
-  fprintf (stderr, "\ngdb: ");
-  vfprintf (stderr, string, args);
-  fprintf (stderr, "\n");
+  fprintf_unfiltered (gdb_stderr, "\ngdb: ");
+  vfprintf_unfiltered (gdb_stderr, string, args);
+  fprintf_unfiltered (gdb_stderr, "\n");
   va_end (args);
   exit (1);
 }
@@ -300,9 +309,9 @@ fatal_dump_core (va_alist)
   string = va_arg (args, char *);
   /* "internal error" is always correct, since GDB should never dump
      core, no matter what the input.  */
-  fprintf (stderr, "\ngdb internal error: ");
-  vfprintf (stderr, string, args);
-  fprintf (stderr, "\n");
+  fprintf_unfiltered (gdb_stderr, "\ngdb internal error: ");
+  vfprintf_unfiltered (gdb_stderr, string, args);
+  fprintf_unfiltered (gdb_stderr, "\n");
   va_end (args);
 
   signal (SIGQUIT, SIG_DFL);
@@ -393,7 +402,10 @@ print_sys_errmsg (string, errcode)
   strcat (combined, ": ");
   strcat (combined, err);
 
-  fprintf (stderr, "%s.\n", combined);
+  /* We want anything which was printed on stdout to come out first, before
+     this message.  */
+  gdb_flush (gdb_stdout);
+  fprintf_unfiltered (gdb_stderr, "%s.\n", combined);
 }
 
 /* Control C eventually causes this to be called, at a convenient time.  */
@@ -401,22 +413,39 @@ print_sys_errmsg (string, errcode)
 void
 quit ()
 {
+  serial_t gdb_stdout_serial = serial_fdopen (1);
+
   target_terminal_ours ();
-  wrap_here ((char *)0);		/* Force out any pending output */
-#if !defined(__GO32__)
-#ifdef HAVE_TERMIO
-  ioctl (fileno (stdout), TCFLSH, 1);
-#else /* not HAVE_TERMIO */
-  ioctl (fileno (stdout), TIOCFLUSH, 0);
-#endif /* not HAVE_TERMIO */
-#ifdef TIOCGPGRP
-  error ("Quit");
-#else
-  error ("Quit (expect signal %d when inferior is resumed)", SIGINT);
-#endif /* TIOCGPGRP */
-#else
-  error ("Quit");
-#endif
+
+  /* We want all output to appear now, before we print "Quit".  We
+     have 3 levels of buffering we have to flush (it's possible that
+     some of these should be changed to flush the lower-level ones
+     too):  */
+
+  /* 1.  The _filtered buffer.  */
+  wrap_here ((char *)0);
+
+  /* 2.  The stdio buffer.  */
+  gdb_flush (gdb_stdout);
+  gdb_flush (gdb_stderr);
+
+  /* 3.  The system-level buffer.  */
+  SERIAL_FLUSH_OUTPUT (gdb_stdout_serial);
+  SERIAL_UN_FDOPEN (gdb_stdout_serial);
+
+  /* Don't use *_filtered; we don't want to prompt the user to continue.  */
+  if (error_pre_print)
+    fprintf_unfiltered (gdb_stderr, error_pre_print);
+
+  if (job_control
+      /* If there is no terminal switching for this target, then we can't
+	 possibly get screwed by the lack of job control.  */
+      || current_target->to_terminal_ours == NULL)
+    fprintf_unfiltered (gdb_stderr, "Quit\n");
+  else
+    fprintf_unfiltered (gdb_stderr,
+	     "Quit (expect signal SIGINT when the program is resumed)\n");
+  return_to_top_level (RETURN_QUIT);
 }
 
 
@@ -431,16 +460,49 @@ pollquit()
   if (kbhit ())
     {
       int k = getkey ();
-      if (k == 1)
+      if (k == 1) {
 	quit_flag = 1;
-      else if (k == 2)
+	quit();
+      }
+      else if (k == 2) {
 	immediate_quit = 1;
-      quit ();
+	quit ();
+      }
+      else 
+	{
+	  /* We just ignore it */
+	  fprintf_unfiltered (gdb_stderr, "CTRL-A to quit, CTRL-B to quit harder\n");
+	}
     }
 }
 
-#endif
 
+#endif
+#ifdef __GO32__
+void notice_quit()
+{
+  if (kbhit ())
+    {
+      int k = getkey ();
+      if (k == 1) {
+	quit_flag = 1;
+      }
+      else if (k == 2)
+	{
+	  immediate_quit = 1;
+	}
+      else 
+	{
+	  fprintf_unfiltered (gdb_stderr, "CTRL-A to quit, CTRL-B to quit harder\n");
+	}
+    }
+}
+#else
+void notice_quit()
+{
+  /* Done by signals */
+}
+#endif
 /* Control C comes here */
 
 void
@@ -609,7 +671,7 @@ PTR
 xmalloc (size)
      long size;
 {
-  return (xmmalloc ((void *) NULL, size));
+  return (xmmalloc ((PTR) NULL, size));
 }
 
 /* Like mrealloc but get error if no storage available.  */
@@ -619,7 +681,7 @@ xrealloc (ptr, size)
      PTR ptr;
      long size;
 {
-  return (xmrealloc ((void *) NULL, ptr, size));
+  return (xmrealloc ((PTR) NULL, ptr, size));
 }
 
 
@@ -665,7 +727,7 @@ savestring (ptr, size)
 
 char *
 msavestring (md, ptr, size)
-     void *md;
+     PTR md;
      const char *ptr;
      int size;
 {
@@ -687,7 +749,7 @@ strsave (ptr)
 
 char *
 mstrsave (md, ptr)
-     void *md;
+     PTR md;
      const char *ptr;
 {
   return (msavestring (md, ptr, strlen (ptr)));
@@ -724,13 +786,13 @@ query (va_alist)
   while (1)
     {
       wrap_here ("");		/* Flush any buffered output */
-      fflush (stdout);
+      gdb_flush (gdb_stdout);
       va_start (args);
       ctlstr = va_arg (args, char *);
-      vfprintf_filtered (stdout, ctlstr, args);
+      vfprintf_filtered (gdb_stdout, ctlstr, args);
       va_end (args);
       printf_filtered ("(y or n) ");
-      fflush (stdout);
+      gdb_flush (gdb_stdout);
       answer = fgetc (stdin);
       clearerr (stdin);		/* in case of C-d */
       if (answer == EOF)	/* C-d */
@@ -899,17 +961,22 @@ static unsigned int lines_printed, chars_printed;
    spit it out and forget about the wrap_here().  If we see another
    wrap_here(), we spit it out and remember the newer one.  If we see
    the end of the line, we spit out a newline, the indent, and then
-   the buffered output.
+   the buffered output.  */
 
-   wrap_column is the column number on the screen where wrap_buffer begins.
-     When wrap_column is zero, wrapping is not in effect.
-   wrap_buffer is malloc'd with chars_per_line+2 bytes. 
-     When wrap_buffer[0] is null, the buffer is empty.
-   wrap_pointer points into it at the next character to fill.
-   wrap_indent is the string that should be used as indentation if the
-     wrap occurs.  */
+/* Malloc'd buffer with chars_per_line+2 bytes.  Contains characters which
+   are waiting to be output (they have already been counted in chars_printed).
+   When wrap_buffer[0] is null, the buffer is empty.  */
+static char *wrap_buffer;
 
-static char *wrap_buffer, *wrap_pointer, *wrap_indent;
+/* Pointer in wrap_buffer to the next character to fill.  */
+static char *wrap_pointer;
+
+/* String to indent by if the wrap occurs.  Must not be NULL if wrap_column
+   is non-zero.  */
+static char *wrap_indent;
+
+/* Column number on the screen where wrap_buffer begins, or 0 if wrapping
+   is not in effect.  */
 static int wrap_column;
 
 /* ARGSUSED */
@@ -981,7 +1048,7 @@ reinitialize_more_filter ()
 
 /* Indicate that if the next sequence of characters overflows the line,
    a newline should be inserted here rather than when it hits the end. 
-   If INDENT is nonzero, it is a string to be printed to indent the
+   If INDENT is non-null, it is a string to be printed to indent the
    wrapped part on the next line.  INDENT must remain accessible until
    the next call to wrap_here() or until a newline is printed through
    fputs_filtered().
@@ -993,17 +1060,21 @@ reinitialize_more_filter ()
    we must not wrap words, but should still keep track of newlines
    that were explicitly printed.
 
-   INDENT should not contain tabs, as that
-   will mess up the char count on the next line.  FIXME.  */
+   INDENT should not contain tabs, as that will mess up the char count
+   on the next line.  FIXME.
+
+   This routine is guaranteed to force out any output which has been
+   squirreled away in the wrap_buffer, so wrap_here ((char *)0) can be
+   used to force out output from the wrap_buffer.  */
 
 void
 wrap_here(indent)
-  char *indent;
+     char *indent;
 {
   if (wrap_buffer[0])
     {
       *wrap_pointer = '\0';
-      fputs (wrap_buffer, stdout);
+      fputs  (wrap_buffer, gdb_stdout);
     }
   wrap_pointer = wrap_buffer;
   wrap_buffer[0] = '\0';
@@ -1014,13 +1085,17 @@ wrap_here(indent)
   else if (chars_printed >= chars_per_line)
     {
       puts_filtered ("\n");
-      puts_filtered (indent);
+      if (indent != NULL)
+	puts_filtered (indent);
       wrap_column = 0;
     }
   else
     {
       wrap_column = chars_printed;
-      wrap_indent = indent;
+      if (indent == NULL)
+	wrap_indent = "";
+      else
+	wrap_indent = indent;
     }
 }
 
@@ -1038,20 +1113,40 @@ begin_line ()
     }
 }
 
-/* Like fputs but pause after every screenful, and can wrap at points
-   other than the final character of a line.
-   Unlike fputs, fputs_filtered does not return a value.
+
+GDB_FILE *
+gdb_fopen (name, mode)
+     char * name;
+     char * mode;
+{
+  return fopen (name, mode);
+}
+
+void
+gdb_flush (stream)
+     FILE *stream;
+{
+  fflush (stream);
+}
+
+/* Like fputs but if FILTER is true, pause after every screenful.
+
+   Regardless of FILTER can wrap at points other than the final
+   character of a line.
+
+   Unlike fputs, fputs_maybe_filtered does not return a value.
    It is OK for LINEBUFFER to be NULL, in which case just don't print
    anything.
 
-   Note that a longjmp to top level may occur in this routine
-   (since prompt_for_continue may do so) so this routine should not be
-   called when cleanups are not in place.  */
+   Note that a longjmp to top level may occur in this routine (only if
+   FILTER is true) (since prompt_for_continue may do so) so this
+   routine should not be called when cleanups are not in place.  */
 
-void
-fputs_filtered (linebuffer, stream)
+static void
+fputs_maybe_filtered (linebuffer, stream, filter)
      const char *linebuffer;
      FILE *stream;
+     int filter;
 {
   const char *lineptr;
 
@@ -1059,7 +1154,7 @@ fputs_filtered (linebuffer, stream)
     return;
   
   /* Don't do any filtering if it is disabled.  */
-  if (stream != stdout
+  if (stream != gdb_stdout
    || (lines_per_page == UINT_MAX && chars_per_line == UINT_MAX))
     {
       fputs (linebuffer, stream);
@@ -1074,7 +1169,8 @@ fputs_filtered (linebuffer, stream)
   while (*lineptr)
     {
       /* Possible new page.  */
-      if (lines_printed >= lines_per_page - 1)
+      if (filter &&
+	  (lines_printed >= lines_per_page - 1))
 	prompt_for_continue ();
 
       while (*lineptr && *lineptr != '\n')
@@ -1121,8 +1217,7 @@ fputs_filtered (linebuffer, stream)
 	      /* Now output indentation and wrapped string */
 	      if (wrap_column)
 		{
-		  if (wrap_indent)
-		    fputs (wrap_indent, stream);
+		  fputs (wrap_indent, stream);
 		  *wrap_pointer = '\0';		/* Null-terminate saved stuff */
 		  fputs (wrap_buffer, stream);	/* and eject it */
 		  /* FIXME, this strlen is what prevents wrap_indent from
@@ -1151,6 +1246,55 @@ fputs_filtered (linebuffer, stream)
     }
 }
 
+void
+fputs_filtered (linebuffer, stream)
+     const char *linebuffer;
+     FILE *stream;
+{
+  fputs_maybe_filtered (linebuffer, stream, 1);
+}
+
+void
+fputs_unfiltered (linebuffer, stream)
+     const char *linebuffer;
+     FILE *stream;
+{
+#if 0
+
+  /* This gets the wrap_buffer buffering wrong when called from
+     gdb_readline (GDB was sometimes failing to print the prompt
+     before reading input).  Even at other times, it seems kind of
+     misguided, especially now that printf_unfiltered doesn't use
+     printf_maybe_filtered.  */
+
+  fputs_maybe_filtered (linebuffer, stream, 0);
+#else
+  fputs (linebuffer, stream);
+#endif
+}
+
+void
+putc_unfiltered (c)
+     int c;
+{
+  char buf[2];
+  buf[0] = c;
+  buf[1] = 0;
+  fputs_unfiltered (buf, gdb_stdout);
+}
+
+void
+fputc_unfiltered (c, stream)
+     int c;
+     FILE * stream;
+{
+  char buf[2];
+  buf[0] = c;
+  buf[1] = 0;
+  fputs_unfiltered (buf, stream);
+}
+
+
 /* Print a variable number of ARGS using format FORMAT.  If this
    information is going to put the amount written (since the last call
    to REINITIALIZE_MORE_FILTER or the last page break) over the page size,
@@ -1176,11 +1320,12 @@ fputs_filtered (linebuffer, stream)
 
 #define	MIN_LINEBUF	255
 
-void
-vfprintf_filtered (stream, format, args)
+static void
+vfprintf_maybe_filtered (stream, format, args, filter)
      FILE *stream;
      char *format;
      va_list args;
+     int filter;
 {
   char line_buf[MIN_LINEBUF+10];
   char *linebuffer = line_buf;
@@ -1198,7 +1343,26 @@ vfprintf_filtered (stream, format, args)
      followed.   */
   vsprintf (linebuffer, format, args);
 
-  fputs_filtered (linebuffer, stream);
+  fputs_maybe_filtered (linebuffer, stream, filter);
+}
+
+
+void
+vfprintf_filtered (stream, format, args)
+     FILE *stream;
+     char *format;
+     va_list args;
+{
+  vfprintf_maybe_filtered (stream, format, args, 1);
+}
+
+void
+vfprintf_unfiltered (stream, format, args)
+     FILE *stream;
+     char *format;
+     va_list args;
+{
+  vfprintf (stream, format, args);
 }
 
 void
@@ -1206,7 +1370,15 @@ vprintf_filtered (format, args)
      char *format;
      va_list args;
 {
-  vfprintf_filtered (stdout, format, args);
+  vfprintf_maybe_filtered (gdb_stdout, format, args, 1);
+}
+
+void
+vprintf_unfiltered (format, args)
+     char *format;
+     va_list args;
+{
+  vfprintf (gdb_stdout, format, args);
 }
 
 /* VARARGS */
@@ -1228,8 +1400,27 @@ fprintf_filtered (va_alist)
   va_end (args);
 }
 
+/* VARARGS */
+void
+fprintf_unfiltered (va_alist)
+     va_dcl
+{
+  va_list args;
+  FILE *stream;
+  char *format;
+
+  va_start (args);
+  stream = va_arg (args, FILE *);
+  format = va_arg (args, char *);
+
+  /* This won't blow up if the restrictions described above are
+     followed.   */
+  vfprintf_unfiltered (stream, format, args);
+  va_end (args);
+}
+
 /* Like fprintf_filtered, but prints it's result indent.
-   Called as fprintfi_filtered (spaces, format, arg1, arg2, ...); */
+   Called as fprintfi_filtered (spaces, stream, format, ...);  */
 
 /* VARARGS */
 void
@@ -1253,6 +1444,7 @@ fprintfi_filtered (va_alist)
   va_end (args);
 }
 
+
 /* VARARGS */
 void
 printf_filtered (va_alist)
@@ -1264,12 +1456,28 @@ printf_filtered (va_alist)
   va_start (args);
   format = va_arg (args, char *);
 
-  vfprintf_filtered (stdout, format, args);
+  vfprintf_filtered (gdb_stdout, format, args);
+  va_end (args);
+}
+
+
+/* VARARGS */
+void
+printf_unfiltered (va_alist)
+     va_dcl
+{
+  va_list args;
+  char *format;
+
+  va_start (args);
+  format = va_arg (args, char *);
+
+  vfprintf_unfiltered (gdb_stdout, format, args);
   va_end (args);
 }
 
 /* Like printf_filtered, but prints it's result indented.
-   Called as printfi_filtered (spaces, format, arg1, arg2, ...); */
+   Called as printfi_filtered (spaces, format, ...);  */
 
 /* VARARGS */
 void
@@ -1283,8 +1491,8 @@ printfi_filtered (va_alist)
   va_start (args);
   spaces = va_arg (args, int);
   format = va_arg (args, char *);
-  print_spaces_filtered (spaces, stdout);
-  vfprintf_filtered (stdout, format, args);
+  print_spaces_filtered (spaces, gdb_stdout);
+  vfprintf_filtered (gdb_stdout, format, args);
   va_end (args);
 }
 
@@ -1297,7 +1505,14 @@ void
 puts_filtered (string)
      char *string;
 {
-  fputs_filtered (string, stdout);
+  fputs_filtered (string, gdb_stdout);
+}
+
+void
+puts_unfiltered (string)
+     char *string;
+{
+  fputs_unfiltered (string, gdb_stdout);
 }
 
 /* Return a pointer to N spaces and a null.  The pointer is good
@@ -1485,7 +1700,7 @@ _initialize_utils ()
 #endif
 #endif
   /* If the output is not a terminal, don't paginate it.  */
-  if (!ISATTY (stdout))
+  if (!ISATTY (gdb_stdout))
     lines_per_page = UINT_MAX;
 
   set_width_command ((char *)NULL, 0, c);
