@@ -327,11 +327,11 @@ size_name (x)
 void
 emit (string, a1, a2, a3, a4, a5)
      char *string;
-     int a1;
-     int a2;
-     int a3;
-     int a4;
-     int a5;
+     char* a1;
+     char* a2;
+     char* a3;
+     char* a4;
+     char* a5;
 {
   int indent_inc = 0;
   int indent_dec = 0;
@@ -511,7 +511,7 @@ void
 info_args (p)
      opcode_entry_type *p;
 {
-  unsigned short *s;
+  unsigned int *s;
 
   int done_one_imm8 = 0;
 
@@ -684,6 +684,7 @@ info_special (p, getdst, nostore, before, nosrc)
     case OPC_cp:
     case OPC_cpb:
     case OPC_cpl:
+    case OPC_bit:
       *nostore = 1;
       *before = 0;
       break;
@@ -742,7 +743,7 @@ info_lvals (p)
      opcode_entry_type *p;
 {
   /* emit code to work out lvalues, if any */
-  unsigned short *i = p->arg_info;
+  unsigned int *i = p->arg_info;
 
   while (*i)
     {
@@ -765,6 +766,7 @@ info_lvals (p)
 	case CLASS_REG_WORD:
 	case CLASS_REG_LONG:
 	case CLASS_REG_BYTE:
+	case CLASS_PR:
 	  break;
 	case CLASS_BA:
 	  emit ("register  int oplval_<name> = get_<ptr_mode>_reg(context,reg_<name>) + (short)(imm_src);\n");
@@ -785,7 +787,7 @@ info_fetch (p, getdst)
      opcode_entry_type *p;
      int getdst;
 {
-  unsigned short *i = p->arg_info;
+  unsigned int *i = p->arg_info;
   int had_src = 0;
 
   allregs = 1;
@@ -1073,7 +1075,7 @@ rotate (p, through_carry, size, left)
 }
 
 static void
-div (p)
+adiv (p)
      opcode_entry_type *p;
 {
   emit ("if (op_src==0)\n");
@@ -1109,6 +1111,24 @@ div (p)
   emit ("}\n");
 }
 
+static void
+dobit (p)
+opcode_entry_type *p;
+{
+  emit("context->zero = (op_dst & (1<<op_src))==0;\n");
+  emit("context->broken_flags = 0;\n");
+}
+static void
+doset (p, v)
+opcode_entry_type*p;
+int v;
+{
+  if (v) 
+    emit (" tmp = op_dst | (1<< op_src);\n");
+  else
+    emit (" tmp = op_dst & ~(1<< op_src);\n");
+}
+
 static void
 mult (p)
      opcode_entry_type *p;
@@ -1317,6 +1337,7 @@ info_docode (p)
       break;
     case OPC_lda:
       emit ("tmp = oplval_src; \n");
+      /*(((oplval_src) & 0xff0000) << 8) | (oplval_src & 0xffff); \n");*/
       break;
     case OPC_ldk:
     case OPC_ld:
@@ -1346,7 +1367,7 @@ info_docode (p)
 
     case OPC_div:
     case OPC_divl:
-      div (p);
+      adiv (p);
       break;
     case OPC_mult:
     case OPC_multl:
@@ -1356,7 +1377,15 @@ info_docode (p)
     case OPC_popl:
       pop (p);
       break;
-
+    case OPC_set:
+      doset (p,1);
+      break;
+    case OPC_res:
+      doset (p,0);
+      break;
+    case OPC_bit:
+      dobit(p);
+      break;
     default:
 
       emit ("tmp = fail(context,%d);\n", p->opcode);
@@ -1370,7 +1399,7 @@ void
 info_store (p)
      opcode_entry_type *p;
 {
-  unsigned short *i = p->arg_info;
+  unsigned int *i = p->arg_info;
 
   while (*i)
     {
@@ -1381,9 +1410,13 @@ info_store (p)
 	{
 	  switch (*i & CLASS_MASK)
 	    {
+	    case CLASS_PR:
+	      emit ("put_<ptr_mode>_reg(context,reg_<name>, tmp);\n");
+	      break;
 	    case CLASS_REG_LONG:
 	    case CLASS_REG_WORD:
 	    case CLASS_REG_BYTE:
+
 	      emit ("put_<size>_reg(context,reg_<name>,tmp);\n");
 	      break;
 	    case CLASS_X:
@@ -1391,6 +1424,7 @@ info_store (p)
 	    case CLASS_DA:
 	    case CLASS_BX:
 	    case CLASS_BA:
+
 	      emit ("put_<size>_<mem>_da(context,oplval_<name>, tmp);\n");
 	      break;
 	    case CLASS_IMM:
@@ -1814,7 +1848,7 @@ int
 info_len_in_words (o)
      opcode_entry_type *o;
 {
-  unsigned short int *p = o->byte_info;
+  unsigned  int *p = o->byte_info;
   int nibs = 0;
 
   while (*p)
